@@ -27,9 +27,10 @@
  *
  */
 #include "conky.h"
-#ifdef BUILD_X11
+#ifdef BUILD_GUI
 #include "fonts.h"
-#endif /* BUILD_X11 */
+#include "gui.h"
+#endif /* BUILD_GUI */
 #include <cmath>
 #include "logging.h"
 #include "nc.h"
@@ -43,6 +44,7 @@
 #include "colours.h"
 #include "common.h"
 #include "conky.h"
+#include "display-output.hh"
 
 struct special_t *specials = nullptr;
 
@@ -57,7 +59,7 @@ conky::range_config_setting<int> default_bar_width(
 conky::range_config_setting<int> default_bar_height(
     "default_bar_height", 0, std::numeric_limits<int>::max(), 6, false);
 
-#ifdef BUILD_X11
+#ifdef BUILD_GUI
 conky::range_config_setting<int> default_graph_width(
     "default_graph_width", 0, std::numeric_limits<int>::max(), 0, false);
 conky::range_config_setting<int> default_graph_height(
@@ -67,7 +69,10 @@ conky::range_config_setting<int> default_gauge_width(
     "default_gauge_width", 0, std::numeric_limits<int>::max(), 40, false);
 conky::range_config_setting<int> default_gauge_height(
     "default_gauge_height", 0, std::numeric_limits<int>::max(), 25, false);
-#endif /* BUILD_X11 */
+
+conky::simple_config_setting<bool> store_graph_data_explicitly(
+    "store_graph_data_explicitly", true, true);
+#endif /* BUILD_GUI */
 
 conky::simple_config_setting<std::string> console_graph_ticks(
     "console_graph_ticks", " ,_,=,#", false);
@@ -114,7 +119,7 @@ struct tab {
  * Scanning arguments to various special text objects
  */
 
-#ifdef BUILD_X11
+#ifdef BUILD_GUI
 const char *scan_gauge(struct text_object *obj, const char *args,
                        double scale) {
   struct gauge *g;
@@ -179,7 +184,7 @@ const char *scan_bar(struct text_object *obj, const char *args, double scale) {
   return args;
 }
 
-#ifdef BUILD_X11
+#ifdef BUILD_GUI
 void scan_font(struct text_object *obj, const char *args) {
   if ((args != nullptr) && (*args != 0)) {
     obj->data.s = strndup(args, DEFAULT_TEXT_BUFFER_SIZE);
@@ -337,7 +342,7 @@ char *scan_graph(struct text_object *obj, const char *args, double defscale) {
 
   return nullptr;
 }
-#endif /* BUILD_X11 */
+#endif /* BUILD_GUI */
 
 /*
  * Printing various special text objects
@@ -384,23 +389,23 @@ void new_gauge_in_shell(struct text_object *obj, char *p,
            gaugevals[round_to_positive_int(usage * 4 / g->scale)]);
 }
 
-#ifdef BUILD_X11
-void new_gauge_in_x11(struct text_object *obj, char *buf, double usage) {
+#ifdef BUILD_GUI
+void new_gauge_in_gui(struct text_object *obj, char *buf, double usage) {
   struct special_t *s = nullptr;
   auto *g = static_cast<struct gauge *>(obj->special_data);
 
-  if (!out_to_x.get(*state)) { return; }
+  if (display_output() == nullptr || !display_output()->graphical()) { return; }
 
   if (g == nullptr) { return; }
 
   s = new_special(buf, GAUGE);
 
   s->arg = usage;
-  s->width = xft_dpi_scale(g->width);
-  s->height = xft_dpi_scale(g->height);
+  s->width = dpi_scale(g->width);
+  s->height = dpi_scale(g->height);
   s->scale = g->scale;
 }
-#endif /* BUILD_X11 */
+#endif /* BUILD_GUI */
 
 void new_gauge(struct text_object *obj, char *p, unsigned int p_max_size,
                double usage) {
@@ -414,22 +419,24 @@ void new_gauge(struct text_object *obj, char *p, unsigned int p_max_size,
     usage = std::min(g->scale, usage);
   }
 
-#ifdef BUILD_X11
-  if (out_to_x.get(*state)) { new_gauge_in_x11(obj, p, usage); }
+#ifdef BUILD_GUI
+  if (display_output() && display_output()->graphical()) {
+    new_gauge_in_gui(obj, p, usage);
+  }
   if (out_to_stdout.get(*state)) {
     new_gauge_in_shell(obj, p, p_max_size, usage);
   }
-#else  /* BUILD_X11 */
+#else  /* BUILD_GUI */
   new_gauge_in_shell(obj, p, p_max_size, usage);
-#endif /* BUILD_X11 */
+#endif /* BUILD_GUI */
 }
 
-#ifdef BUILD_X11
+#ifdef BUILD_GUI
 void new_font(struct text_object *obj, char *p, unsigned int p_max_size) {
   struct special_t *s;
   unsigned int tmp = selected_font;
 
-  if (!out_to_x.get(*state)) { return; }
+  if (display_output() == nullptr || !display_output()->graphical()) { return; }
 
   if (p_max_size == 0) { return; }
 
@@ -551,7 +558,7 @@ void new_graph(struct text_object *obj, char *buf, int buf_max_size,
   s = new_special(buf, GRAPH);
 
   /* set graph (special) width to width in obj */
-  s->width = xft_dpi_scale(g->width);
+  s->width = dpi_scale(g->width);
   if (s->width != 0) { s->graph_width = s->width; }
 
   if (s->graph_width != s->graph_allocated) {
@@ -576,7 +583,7 @@ void new_graph(struct text_object *obj, char *buf, int buf_max_size,
     s->graph_allocated = s->graph_width;
     graphs[g->id] = graph;
   }
-  s->height = xft_dpi_scale(g->height);
+  s->height = dpi_scale(g->height);
   s->first_colour = adjust_colours(g->first_colour);
   s->last_colour = adjust_colours(g->last_colour);
   if (g->scale != 0) {
@@ -596,22 +603,25 @@ void new_graph(struct text_object *obj, char *buf, int buf_max_size,
   }
 #endif
 
-  int graph_id = ((struct graph *)obj->special_data)->id;
-  s->graph = retrieve_graph(graph_id, s->graph_width);
+  if (store_graph_data_explicitly.get(*state)) {
+    if (s->graph) { s->graph = retrieve_graph(g->id, s->graph_width); }
 
-  graph_append(s, val, g->flags);
+    graph_append(s, val, g->flags);
 
-  store_graph(graph_id, s);
+    store_graph(g->id, s);
+  } else {
+    graph_append(s, val, g->flags);
+  }
 
   if (out_to_stdout.get(*state)) { new_graph_in_shell(s, buf, buf_max_size); }
 }
 
 void new_hr(struct text_object *obj, char *p, unsigned int p_max_size) {
-  if (!out_to_x.get(*state)) { return; }
+  if (display_output() == nullptr || !display_output()->graphical()) { return; }
 
   if (p_max_size == 0) { return; }
 
-  new_special(p, HORIZONTAL_LINE)->height = xft_dpi_scale(obj->data.l);
+  new_special(p, HORIZONTAL_LINE)->height = dpi_scale(obj->data.l);
 }
 
 void scan_stippled_hr(struct text_object *obj, const char *arg) {
@@ -637,21 +647,23 @@ void new_stippled_hr(struct text_object *obj, char *p,
   struct special_t *s = nullptr;
   auto *sh = static_cast<struct stippled_hr *>(obj->special_data);
 
-  if (!out_to_x.get(*state)) { return; }
+  if (display_output() == nullptr || !display_output()->graphical()) { return; }
 
   if ((sh == nullptr) || (p_max_size == 0)) { return; }
 
   s = new_special(p, STIPPLED_HR);
 
-  s->height = xft_dpi_scale(sh->height);
-  s->arg = xft_dpi_scale(sh->arg);
+  s->height = dpi_scale(sh->height);
+  s->arg = dpi_scale(sh->arg);
 }
-#endif /* BUILD_X11 */
+#endif /* BUILD_GUI */
 
 void new_fg(struct text_object *obj, char *p, unsigned int p_max_size) {
-#ifdef BUILD_X11
-  if (out_to_x.get(*state)) { new_special(p, FG)->arg = obj->data.l; }
-#endif /* BUILD_X11 */
+#ifdef BUILD_GUI
+  if (display_output() && display_output()->graphical()) {
+    new_special(p, FG)->arg = obj->data.l;
+  }
+#endif /* BUILD_GUI */
 #ifdef BUILD_NCURSES
   if (out_to_ncurses.get(*state)) { new_special(p, FG)->arg = obj->data.l; }
 #endif /* BUILD_NCURSES */
@@ -660,15 +672,15 @@ void new_fg(struct text_object *obj, char *p, unsigned int p_max_size) {
   UNUSED(p_max_size);
 }
 
-#ifdef BUILD_X11
+#ifdef BUILD_GUI
 void new_bg(struct text_object *obj, char *p, unsigned int p_max_size) {
-  if (!out_to_x.get(*state)) { return; }
+  if (display_output() == nullptr || !display_output()->graphical()) { return; }
 
   if (p_max_size == 0) { return; }
 
   new_special(p, BG)->arg = obj->data.l;
 }
-#endif /* BUILD_X11 */
+#endif /* BUILD_GUI */
 
 static void new_bar_in_shell(struct text_object *obj, char *buffer,
                              unsigned int buf_max_size, double usage) {
@@ -693,23 +705,23 @@ static void new_bar_in_shell(struct text_object *obj, char *buffer,
   buffer[i] = 0;
 }
 
-#ifdef BUILD_X11
-static void new_bar_in_x11(struct text_object *obj, char *buf, double usage) {
+#ifdef BUILD_GUI
+static void new_bar_in_gui(struct text_object *obj, char *buf, double usage) {
   struct special_t *s = nullptr;
   auto *b = static_cast<struct bar *>(obj->special_data);
 
-  if (!out_to_x.get(*state)) { return; }
+  if (display_output() == nullptr || !display_output()->graphical()) { return; }
 
   if (b == nullptr) { return; }
 
   s = new_special(buf, BAR);
 
   s->arg = usage;
-  s->width = xft_dpi_scale(b->width);
-  s->height = xft_dpi_scale(b->height);
+  s->width = dpi_scale(b->width);
+  s->height = dpi_scale(b->height);
   s->scale = b->scale;
 }
-#endif /* BUILD_X11 */
+#endif /* BUILD_GUI */
 
 /* usage is in range [0,255] */
 void new_bar(struct text_object *obj, char *p, unsigned int p_max_size,
@@ -724,14 +736,16 @@ void new_bar(struct text_object *obj, char *p, unsigned int p_max_size,
     usage = std::min(b->scale, usage);
   }
 
-#ifdef BUILD_X11
-  if (out_to_x.get(*state)) { new_bar_in_x11(obj, p, usage); }
+#ifdef BUILD_GUI
+  if (display_output() && display_output()->graphical()) {
+    new_bar_in_gui(obj, p, usage);
+  }
   if (out_to_stdout.get(*state)) {
     new_bar_in_shell(obj, p, p_max_size, usage);
   }
-#else  /* BUILD_X11 */
+#else  /* BUILD_GUI */
   new_bar_in_shell(obj, p, p_max_size, usage);
-#endif /* BUILD_X11 */
+#endif /* BUILD_GUI */
 }
 
 void new_outline(struct text_object *obj, char *p, unsigned int p_max_size) {
@@ -741,12 +755,12 @@ void new_outline(struct text_object *obj, char *p, unsigned int p_max_size) {
 
 void new_offset(struct text_object *obj, char *p, unsigned int p_max_size) {
   if (p_max_size == 0) { return; }
-  new_special(p, OFFSET)->arg = xft_dpi_scale(obj->data.l);
+  new_special(p, OFFSET)->arg = dpi_scale(obj->data.l);
 }
 
 void new_voffset(struct text_object *obj, char *p, unsigned int p_max_size) {
   if (p_max_size == 0) { return; }
-  new_special(p, VOFFSET)->arg = xft_dpi_scale(obj->data.l);
+  new_special(p, VOFFSET)->arg = dpi_scale(obj->data.l);
 }
 
 void new_save_coordinates(struct text_object *obj, char *p,
@@ -757,18 +771,18 @@ void new_save_coordinates(struct text_object *obj, char *p,
 
 void new_alignr(struct text_object *obj, char *p, unsigned int p_max_size) {
   if (p_max_size == 0) { return; }
-  new_special(p, ALIGNR)->arg = xft_dpi_scale(obj->data.l);
+  new_special(p, ALIGNR)->arg = dpi_scale(obj->data.l);
 }
 
 // A positive offset pushes the text further left
 void new_alignc(struct text_object *obj, char *p, unsigned int p_max_size) {
   if (p_max_size == 0) { return; }
-  new_special(p, ALIGNC)->arg = xft_dpi_scale(obj->data.l);
+  new_special(p, ALIGNC)->arg = dpi_scale(obj->data.l);
 }
 
 void new_goto(struct text_object *obj, char *p, unsigned int p_max_size) {
   if (p_max_size == 0) { return; }
-  new_special(p, GOTO)->arg = xft_dpi_scale(obj->data.l);
+  new_special(p, GOTO)->arg = dpi_scale(obj->data.l);
 }
 
 void scan_tab(struct text_object *obj, const char *arg) {
@@ -796,8 +810,8 @@ void new_tab(struct text_object *obj, char *p, unsigned int p_max_size) {
   if ((t == nullptr) || (p_max_size == 0)) { return; }
 
   s = new_special(p, TAB);
-  s->width = xft_dpi_scale(t->width);
-  s->arg = xft_dpi_scale(t->arg);
+  s->width = dpi_scale(t->width);
+  s->arg = dpi_scale(t->arg);
 }
 
 void clear_stored_graphs() {
